{ ------------------------------------------------------------------------------------------------------------------- }
{                                                                                                                     }
{ Die Unit stellt den Datentypen TIntBaum zur Verfgung.                                                              }
{                                                                                                                     }
{  type TIntBinBaum = class(TBinBaum)                                                                                 }
{         public                                                                                                      }
{           Wurzelinhalt: integer;                                                                                    }
{           constructor Create(w: Integer);                                                                           }
{           function WurzelinhaltToString: string; override;                                                          }
{       end;                                                                                                          }
{                                                                                                                     }
{  Ein Binrbaum kann entweder leer sein (NIL) oder eine Wurzel mit zwei evtl. leeren Teilbumen rtb, ltb             }
{                                                                                                                     }
{ --------------------------------------------------------------- Anwendung zum Tool Binrbaum, (C) 2003 D. Garmann - }

unit Intbaum;

interface
  uses SysUtils, StdCtrls, BinBaum;

  type TIntBinBaum = class(TBinBaum)
         public
           Wurzelinhalt: integer;
           constructor Create(w: Integer);
           function WurzelinhaltToString: string; override;
       end;

  function VonPreOrder(var s: string): TIntBinBaum;
  procedure BaumInMemo(b: TBinBaum; m: TMemo);
  function BaumHoehe(b: TBinBaum): integer;
  function KnotenAnzahl(b: TBinBaum): integer;
  function BlattAnzahl(b: TBinBaum): integer;
  function IstSuchbaum(b: TIntBinBaum): boolean;
  function IstAVLBaum(b: TBinBaum): boolean;



implementation

constructor TIntBinBaum.Create(w: integer);
begin
  inherited create;
  Wurzelinhalt:= w;
end;

function TIntBinBaum.WurzelinhaltToString: string;
begin
  WurzelInhaltToString:= IntToStr(Wurzelinhalt);
end;


function VonPreOrder(var s: string): TIntBinBaum;
var ltb, rtb: TIntBinBaum;
    b: TIntBinBaum;
    z: string;
    w: integer;
begin
  if s=''
  then b:= nil
  else begin
         if copy(s,1,1)='#'
         then begin
                b:= nil;
                if pos(',',s)=0
                then s:= ''
                else delete(s,1,pos(',',s));
              end
         else begin
                z:= copy(s,1,pos(',',s)-1);
                if z='' then z:= s;
                w:= StrToInt(z);
                b:= TIntBinBaum.Create(w);
                if pos(',',s)=0
                then s:= ''
                else delete(s,1,pos(',',s));
                ltb:= VonPreOrder(s);
                rtb:= VonPreOrder(s);
                b.FuegeTeilbaumLinksAn(ltb);
                b.FuegeTeilbaumRechtsAn(rtb);
              end;
       end;
  VonPreOrder:= b;
end;

procedure AusgabeRekursiv(b: TBinBaum; m: TMemo; Tiefe: integer);
var z: string;
    i: integer;
begin
  if not b.leer
  then begin
         AusgabeRekursiv(b.RechterTeilbaum,m,Tiefe+1);
         z:= '';
         for i:= 1 to Tiefe
         do z:= z+ '  ';
         z:= z+ b.WurzelinhaltToString;
         m.Lines.Add(z);
         AusgabeRekursiv(b.LinkerTeilbaum,m,Tiefe+1);
       end;
end;

procedure BaumInMemo(b: TBinBaum; m: TMemo);
begin
  AusgabeRekursiv(b,m,0);
end;

function BaumHoehe(b: TBinBaum): integer;
var bhl, bhr: integer;
begin
  if b.leer
  then BaumHoehe:= 0
  else begin
         bhl:= BaumHoehe(b.LinkerTeilbaum);
         bhr:= BaumHoehe(b.RechterTeilbaum);
         if bhl>bhr
         then BaumHoehe:= bhl+1
         else BaumHoehe:= bhr+1;
       end;
end;

function KnotenAnzahl(b: TBinBaum): integer;
var kal, kar: integer;
begin
  if b.leer
  then KnotenAnzahl:= 0
  else begin
         kal:= KnotenAnzahl(b.LinkerTeilbaum);
         kar:= KnotenAnzahl(b.RechterTeilbaum);
         KnotenAnzahl:= kal+kar+1;
       end;
end;

function BlattAnzahl(b: TBinBaum): integer;
var bal, bar: integer;
begin
  if b.leer
  then BlattAnzahl:= 0
  else if b.istBlatt
       then BlattAnzahl:= 1
       else begin
              bal:= BlattAnzahl(b.LinkerTeilbaum);
              bar:= BlattAnzahl(b.RechterTeilbaum);
              BlattAnzahl:= bal+bar;
            end;
end;

function GroesstesElement(b: TIntBinBaum): integer;
var g, lg, rg: integer;
begin
  if b.leer
  then GroesstesElement:= -maxInt
  else begin
         g:= b.Wurzelinhalt;
         lg:= GroesstesElement(b.LinkerTeilbaum as TIntBinBaum);
         rg:= GroesstesElement(b.RechterTeilbaum as TIntBinBaum);
         if (lg>rg)
         then begin
                if (lg>g)
                then GroesstesElement:= lg
                else GroesstesElement:= g;
              end
         else begin
                if (rg>g)
                then GroesstesElement:= rg
                else GroesstesElement:= g;
              end;
       end;
end;

function KleinstesElement(b: TIntBinBaum): integer;
var k, lk, rk: integer;
begin
  if b.leer
  then KleinstesElement:= maxInt
  else begin
         k:= b.Wurzelinhalt;
         lk:= KleinstesElement(b.LinkerTeilbaum as TIntBinBaum);
         rk:= KleinstesElement(b.RechterTeilbaum as TIntBinBaum);
         if (lk<rk)
         then begin
                if (lk<k)
                then KleinstesElement:= lk
                else KleinstesElement:= k;
              end
         else begin
                if (rk<k)
                then KleinstesElement:= rk
                else KleinstesElement:= k;
              end;
       end;
end;

function IstSuchbaum(b: TIntBinBaum): boolean;
begin
  if b.leer
  then IstSuchbaum:= true
  else begin
         IstSuchbaum:=
           (GroesstesElement(b.LinkerTeilbaum as TIntBinBaum) < b.Wurzelinhalt) and
           (KleinstesElement(b.RechterTeilbaum as TIntBinBaum) > b.Wurzelinhalt) and
           (IstSuchbaum(b.LinkerTeilbaum as TIntBinBaum)) and
           (IstSuchbaum(b.RechterTeilbaum as TIntBinBaum));
       end;
end;

function IstAVLBaum(b: TBinBaum): boolean;
var bhl, bhr: integer;
begin
  if b.leer
  then IstAVLBaum:= true
  else begin
         bhl:= BaumHoehe(b.LinkerTeilbaum);
         bhr:= BaumHoehe(b.RechterTeilbaum);
         IstAVLBaum:=
           (abs(bhl-bhr)<=1) and
           IstAVLBaum(b.LinkerTeilbaum) and
           IstAVLBaum(b.RechterTeilbaum);
       end;
end;


end.
