{ ------------------------------------------------------------------------------------------------------------------- }
{                                                                                                                     }
{ Die Unit stellt einen inhaltslosten Datentypen TBinBaum zur Verfgung. Um einen Binrbaum mit Inhalt zu erhalten,   }
{ muss eine hiervon abgeleitete Klasse erstellt werden, z. B.:                                                        }
{                                                                                                                     }
{  type TIntBinBaum = class(TBinBaum)                                                                                 }
{         public                                                                                                      }
{           Wurzelinhalt: integer;                                                                                    }
{           constructor Create(w: Integer);                                                                           }
{           function WurzelinhaltToString: string; override;                                                          }
{       end;                                                                                                          }
{                                                                                                                     }
{  Ein Binrbaum kann entweder leer sein (NIL) oder eine Wurzel mit zwei evtl. leeren Teilbumen rtb, ltb             }
{                                                                                                                     }
{ ------------------------------------------------------------------------- Tool zum Binrbaum, (C) 2001 D. Garmann - }

unit Binbaum;

interface

type
      TBinBaum = class
         private
         ltb, rtb: TBinBaum;
         public
         constructor Create;
                                { Erzeugt ein Blatt, d.h. "leeres Element mit rtb=ltb=NIL                             }
         destructor Destroy;
                                { Zerstrt das Element inklusive seiner Teilbume                                     }
         function Leer: boolean;
                                { Liefert "true" wenn der Baum leer ist, d. h. der Baum ist = NIL                     }
         function WurzelinhaltToString: string; virtual; abstract;
                                { Abstracte Methode zur Darstellung des Inhalts auf dem Bildschirm (Editfeld, etc.)   }

         function LinkerTeilbaum: TBinBaum;
                                { Liefert den Linken Teilbaum im Funktionsergebnis zurck.                            }
         function RechterTeilbaum: TBinBaum;
                                { Liefert den Rechten Teilbaum im Funktionsergebnis zurck.                           }

         procedure FuegeTeilbaumLinksAn(tb: TBinBaum);
                                { Fgt einen Teilbaum an das aktuelle Baumelement links an.                           }
                                { VORSICHT! Ein eventuell vorhandener linker Teilbaum geht verloren!                  }
         procedure FuegeTeilbaumRechtsAn(tb: TBinBaum);
                                { Fgt einen Teilbaum an das aktuelle Baumelement rechts an.                          }
                                { VORSICHT! Ein eventuell vorhandener rechter Teilbaum geht verloren!                 }

         function  LinksLeer: Boolean;
                                { Liefert "true" wenn der linke Teilbaum leer ist, d. h. ltb=NIL                      }
                                { VORSICHT! Ein leerer Baum hat keinen linken Teilbaum => Laufzeitfehler              }
         function  RechtsLeer: Boolean;
                                { Liefert "true" wenn der rechte Teilbaum leer ist, d. h. rtb=NIL                     }
                                { VORSICHT! Ein leerer Baum hat keinen rechten Teilbaum => Laufzeitfehler             }
         function  IstBlatt: Boolean;
                                { Liefert "true", wenn das aktuelle Element leere Teilbume hat.                      }

         function ErsetztDurchLinkenTeilbaum: TBinBaum;
                                { Liefert den Linken Teilbaum im Funktionsergebnis zurck und LSCHT den Rest.        }
         function ErsetztDurchRechtenTeilbaum: TBinBaum;
                                { Liefert den Rechten Teilbaum im Funktionsergebnis zurck und LSCHT den Rest.       }
       end;

implementation

constructor TBinBaum.Create;
begin
  rtb:= NIL;                                             { Rechter und linker Teilbaum auf NIL, einen Inhalt gibt     }
  ltb:= NIL;                                             { es auf dieser Hierarchieebene noch nicht.                  }
end;

destructor TBinBaum.Destroy;
begin
  ltb.Free;                                              { wenn noch links etwas ist, dann dies zerstren             }
  rtb.Free;                                              { wenn noch rechts etwas ist, dann auch das zerstren        }
  inherited Destroy;                                     { und schlielich sich selbst (ererbte Methode von TObject)  }
end;

function TBinBaum.Leer: boolean;
begin
  Leer:= (Self=NIL);                                     { Leer heit, dass der Zeiger "Self" auf NIL zeigt           }
end;

function TBinBaum.LinkerTeilbaum: TBinBaum;
begin
  LinkerTeilbaum:= ltb;                                  { klar                                                       }
end;

function TBinBaum.RechterTeilbaum: TBinBaum;
begin
  RechterTeilbaum:= rtb;                                 { klar                                                       }
end;

procedure TBinBaum.FuegeTeilbaumLinksAn(tb: TBinBaum);
begin
  ltb:= tb;                                              { klar                                                       }
end;

procedure TBinBaum.FuegeTeilbaumRechtsAn(tb: TBinBaum);
begin
  rtb:= tb;                                              { klar                                                       }
end;

function  TBinBaum.LinksLeer: Boolean;
begin
  LinksLeer:= ltb.Leer;                                  { klar                                                       }
end;

function  TBinBaum.RechtsLeer: Boolean;
begin
  RechtsLeer:= rtb.Leer;                                 { klar                                                       }
end;

function  TBinBaum.IstBlatt: Boolean;
begin
  IstBlatt:= rtb.Leer and ltb.Leer;                      { klar                                                       }
end;

function TBinBaum.ErsetztDurchLinkenTeilbaum: TBinBaum;
var h: TBinBaum;
begin
  h:= LinkerTeilbaum;                                    { Linken Teilbaum holen. Dieser wird spter zurckgegeben.   }
  FuegeTeilbaumLinksAn(NIL);                             { Linken Teilbaum abhngen, d. h. leeren Teilbaum links dran }
  Free;                                                  { kompletten Baum zerstren. Linker TB war ja gerettet.      }
  ErsetztDurchLinkenTeilbaum:= h;                        { Dieser kann nun als Resultat zurckgegeben werden.         }
end;

function TBinBaum.ErsetztDurchRechtenTeilbaum: TBinBaum;
var h: TBinBaum;
begin
  h:= RechterTeilbaum;                                   { Rechten Teilbaum holen. Dieser wird spter zurckgegeben.  }
  FuegeTeilbaumRechtsAn(NIL);                            { Isolieren                                                  }
  Free;                                                  { kompletten Baum zerstren                                  }
  ErsetztDurchRechtenTeilbaum:= h;                       { Geretteten rechten TB zurckgeben.                         }
end;

end.
